package game.gui.world;

import java.io.File;
import java.text.SimpleDateFormat;
import java.util.Collections;
import java.util.Date;
import game.color.TextColor;
import game.gui.GuiMenu;
import game.gui.element.ActButton;
import game.gui.element.ActButton.Mode;
import game.gui.element.GuiList;
import game.gui.element.ListEntry;
import game.gui.element.NavButton;
import game.gui.server.GuiConnect;
import game.init.Config;
import game.log.Log;
import game.network.Player;
import game.renderer.Drawing;
import game.util.FileUtils;
import game.util.Tuple;
import game.util.Util;

public class GuiServers extends GuiList<GuiServers.ServerInfo> implements ActButton.Callback {
	public class ServerInfo implements Comparable<ServerInfo>, ListEntry {
		private String name;
		private String address;
		private int port;
		private String user;
		private String password;
		private String access;
		private long lastConnected;

		public ServerInfo(String name, String address, int port, String user, String password, String access, long lastConnected) {
			this.name = name;
			this.address = address;
			this.port = port;
			this.user = user;
			this.password = password;
			this.access = access;
			this.lastConnected = lastConnected;
		}

		public String getName() {
			return this.name;
		}

		public String getAddress() {
			return this.address;
		}

		public int getPort() {
			return this.port;
		}

		public String getUser() {
			return this.user;
		}

		public String getPassword() {
			return this.password;
		}

		public String getAccess() {
			return this.access;
		}

		public long getLastConnected() {
			return this.lastConnected;
		}

		public void setData(String name, String address, int port, String user, String password, String access) {
			this.name = name;
			this.address = address;
			this.port = port;
			this.user = user;
			this.password = password;
			this.access = access;
		}

		public void setLastConnected() {
			this.lastConnected = System.currentTimeMillis();
		}

		public int compareTo(ServerInfo comp) {
			return this.lastConnected < comp.lastConnected ? 1 : (this.lastConnected > comp.lastConnected ? -1 : this.name.compareTo(comp.name));
		}

		public void select(boolean isDoubleClick, int mouseX, int mouseY) {
			GuiServers.this.selectButton.enabled = true;
			GuiServers.this.deleteButton.enabled = true;
			GuiServers.this.editButton.enabled = true;
			GuiServers.this.copyButton.enabled = true;

			if(isDoubleClick) {
				GuiServers.this.use(GuiServers.this.selectButton, Mode.PRIMARY);
			}
		}

		public void draw(int x, int y, int mouseXIn, int mouseYIn, boolean hover) {
			Drawing.drawText(this.name + TextColor.GRAY + " - " + TextColor.RESET + this.user, x + 2, y, 0xffffffff);
			Drawing.drawText(this.address + TextColor.GRAY + " Port " + TextColor.RESET + this.port, x + 2, y + 18, 0xff808080);
			Drawing.drawText("Zuletzt verbunden: " + (this.lastConnected == -1L ? "nie" : DATE_FORMAT.format(new Date(this.lastConnected))), x + 2, y + 18 + 16, 0xff808080);
		}
	}

	public static final GuiServers INSTANCE = new GuiServers();
	private static final SimpleDateFormat DATE_FORMAT = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss");
	private static final File SERVERS_FILE = new File("servers.cfg");

	private ActButton deleteButton;
	private ActButton selectButton;
	private ActButton copyButton;
	private ActButton editButton;
	private ActButton createButton;

	private GuiServers() {
	}

	public void init(int width, int height) {
		super.init(width, height);
		this.setDimensions(width, height, 32, height - 60);
		this.elements.clear();
		if(SERVERS_FILE.exists()) {
			try {
				String[] lines = FileUtils.read(SERVERS_FILE).split("\n");
				String name = "";
				String address = "";
				int port = -1;
				String user = "";
				String password = "";
				String access = "";
				long time = -1L;
				for(int z = 0; z <= lines.length; z++) {
					String line = z == lines.length ? null : lines[z];
					if(line == null || (line.startsWith("[") && line.endsWith("]"))) {
						if(!name.isEmpty() && !address.isEmpty() && !user.isEmpty() && user.length() < Player.MAX_USER_LENGTH && Player.isValidUser(user) && password.length() < Player.MAX_PASS_LENGTH && access.length() < Player.MAX_PASS_LENGTH && address.length() < 128 && name.length() < 128 && port >= 0 && port < 65536)
							this.elements.add(new ServerInfo(name, address, port, user, password, access, time));
						if(line != null) {
							address = "";
							port = -1;
							user = "";
							password = "";
							access = "";
							time = -1L;
							name = line.substring(1, line.length() - 1);
						}
					}
					else {
						Tuple<String, String> value = Util.getKeyValue(line);
						if(value.first.equals("address"))
							address = value.second;
						else if(value.first.equals("port"))
							try {
								port = Integer.parseInt(value.second);
							}
							catch(NumberFormatException e) {
							}
						else if(value.first.equals("user"))
							user = value.second;
						else if(value.first.equals("password"))
							password = value.second;
						else if(value.first.equals("access"))
							access = value.second;
						else if(value.first.equals("connected"))
							try {
								time = Long.parseLong(value.second);
							}
							catch(NumberFormatException e) {
							}
					}
				}
				Collections.sort(this.elements);
			}
			catch(Exception e) {
				Log.IO.error("Konnte Serverliste nicht laden", e);
				this.elements.clear();
			}
		}

		this.add(this.selectButton = new ActButton(width / 2 - 383, height - 56, 150, 24, this, "Verbinden"));
		this.add(this.createButton = new ActButton(width / 2 + 233, height - 56, 150, 24, this, "Hinzufügen ..."));
		this.add(this.deleteButton = new ActButton(width / 2 - 229, height - 28, 150, 24, this, "Löschen"));
        this.add(this.editButton = new ActButton(width / 2 - 75, height - 28, 150, 24, this, "Bearbeiten"));
		this.add(this.copyButton = new ActButton(width / 2 - 383, height - 28, 150, 24, this, "Kopieren"));
        this.add(new NavButton(width / 2 - 75, height - 56, 150, 24, GuiConnect.INSTANCE, "Schnellverbdg."));
		this.add(new NavButton(width / 2 + 233, height - 28, 150, 24, GuiMenu.INSTANCE, "Abbrechen"));

		this.selectButton.enabled = false;
		this.deleteButton.enabled = false;
		this.editButton.enabled = false;
		this.copyButton.enabled = false;
	}
	
	public void onGuiClosed() {
		this.save();
    }
	
	public void applyServer(ServerInfo server) {
		if(this.selectedElement < 0)
			this.elements.add(server);
		this.save();
		this.gm.displayGuiScreen(this);
	}
	
	private void save() {
		try {
			StringBuilder sb = new StringBuilder();
			for(ServerInfo server : this.elements) {
				if(sb.length() > 0)
					sb.append("\n");
				sb.append("[" + server.getName() + "]\n");
				sb.append("address " + server.getAddress() + "\n");
				sb.append("port " + server.getPort() + "\n");
				sb.append("user " + server.getUser() + "\n");
				sb.append("password " + server.getPassword() + "\n");
				sb.append("access " + server.getAccess() + "\n");
				sb.append("connected " + server.getLastConnected());
			}
			FileUtils.write(SERVERS_FILE, sb.toString());
		}
		catch(Exception e) {
			Log.IO.error("Konnte Serverliste nicht speichern", e);
		}
	}

	public String getTitle() {
		return "Server auswählen";
	}
	
	public int getListWidth() {
		return 660;
	}

	public int getSlotHeight() {
		return 56;
	}

	public void use(ActButton button, Mode mode) {
		if(button == this.deleteButton) {
			if(this.selectedElement >= 0) {
				this.elements.remove(this.selectedElement);
				this.gm.displayGuiScreen(this);
			}
		}
		else if(button == this.selectButton) {
			ServerInfo server = this.getSelected();
			if(server != null) {
				server.setLastConnected();
				this.gm.connect(server.address, server.port, server.user, server.password, server.access);
			}
		}
		else if(button == this.createButton) {
			this.setSelected(-1);
			this.gm.displayGuiScreen(new GuiConnect(new ServerInfo("", "", Config.PORT, "", "", "", -1L)));
		}
		else if(button == this.editButton) {
			ServerInfo server = this.getSelected();
			if(server != null)
				this.gm.displayGuiScreen(new GuiConnect(server));
		}
		else if(button == this.copyButton) {
			ServerInfo server = this.getSelected();
			if(server != null) {
				this.setSelected(-1);
				this.gm.displayGuiScreen(new GuiConnect(new ServerInfo(server.name, server.address, server.port, server.user, server.address, server.access, -1L)));
			}
		}
	}
}
